{% extends "django_datawatch/base.html" %}
{% load i18n %}
{% load django_bootstrap5 %}

{% block django_datawatch %}
    <div class="container mt-2 mb-4">
        <div class="card">
            <div class="card-body">
                <div>
                    <form method="post" class="form-horizontal">
                        {% csrf_token %}
                        {% bootstrap_form form layout='horizontal' %}
                        <div class="offset-md-2 col-md-10">
                            {% bootstrap_button action button_type="submit" button_class="btn-primary" %}
                            &#160;
                            <a href="javascript:history.back();">{% trans "Cancel" %}</a>
                        </div>
                    </form>
                </div>
            </div>
        </div>
    </div>
{% endblock %}
