# Generated by Django 4.2.9 on 2024-01-16 00:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_datawatch', '0005_migrate_new_assigned_fields'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='result',
            name='assigned_to_group',
        ),
        migrations.RemoveField(
            model_name='result',
            name='assigned_to_user',
        ),
    ]
